#!/usr/bin/perl
#
# incident.pl v1.8 - Script to handle security incidents
# Copyright (C) 2001  Viraj Alankar
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use Socket;
use Net::Telnet;
use POSIX qw(strftime);
use Mail::Sender;
use Getopt::Std;
use Net::Rwhois;
use Net::Rwhois::ResultSet;
use Net::Rwhois::Connection;
use Net::Rwhois::Query;
use Net::Rwhois::QueryResult;
use Net::Rwhois::Referral;
use Net::Rwhois::ReferralSet;
use Net::Rwhois::RwhoisObject;
use Net::Rwhois::WhoisQuery;
use Net::Rwhois::WhoisResult;

# CONFIG SECTION
################

# NOTE: Below are default values. You can either change these or 
# create an $HOME/.incident.pl.cfg or ./incident.pl.cfg (in the current directory)
# All of the values below will be overridden by a configuration file.
# See incident.pl.cfg.sample that came with this script.

# For email contacts, always include these users of the suspect domain
@admin_emails = ( "root", "postmaster", "webmaster", "abuse", "security", "noc");

# EMAIL information
$MYDOMAIN = "mydomain.com";
$FROM_ADDR = 'Full Name <youremail@address.com>';
$SMTP_SERVER = "your.mail.server";
$ALWAYS_CC = 'someone@whatever.blah';

# Your timezone information, which is by default included in the message body
$TIMEZONE_STR = "Eastern US time - GMT -0400";

# Rwhois server for name lookups
$RWHOIS_SERVER = 'root.rwhois.net';

# Rwhois server for IP lookups
$RWHOIS_ARIN_SERVER = 'rwhois.arin.net';

# Rwhois port number
$RWHOIS_PORT = 4321;

# Number of seconds to wait for whois results
$WHOIS_TIMEOUT = 60;

# Number of times to try whois if it fails
$WHOIS_NUM_TRIES = 3;

# Number of seconds to wait between whois retries
# Do not make this too low because too many fast queries can get you banned
$WHOIS_RETRY_INTERVAL = 10;

# Number of seconds to wait for rwhois results
$RWHOIS_TIMEOUT = $WHOIS_TIMEOUT;

# Number of times to try rwhois if it fails
$RWHOIS_NUM_TRIES = $WHOIS_NUM_TRIES;

# Number of seconds to wait between rwhois retries
# Do not make this too low because too many fast queries can get you banned
$RWHOIS_RETRY_INTERVAL = $WHOIS_RETRY_INTERVAL;

# Format of how your whois command is called
# Default format is for Linux fwhois
# For Solaris, you may need to change this to 'whois -h SERVER QUERY'
$WHOIS_FORMAT = 'whois QUERY@SERVER';

# TCP services to check for on suspect host
@tcp_ports = ( 21, 22, 23, 25, 79, 80, 139, 6000, 6001 );

# TCP timeout (for telnet)
$TCP_TIMEOUT = 5;

# Override initial string timeouts for these ports
# Basically we want to wait longer for initial string data on these ports
# if there is connection to the port within $TCP_TIMEOUT
%tcp_override_timeouts = (
	"21" => "30",
	"22" => "30",
	"23" => "30",
	"25" => "30"
);

# Reporting directory
# If not defined, will use current directory
# $REP_DIR = "/home/valankar/incident/incidents";

# Report file (in subdir of $REP_DIR)
$REP_FILE = "orig_email.txt";

# Will store email addresses of who was sent $REP_FILE (in subdir of $REP_DIR)
$SENT_TO_FILE = "orig_email_sent_to.txt";

# Where to store email followups (in subdir of $REP_DIR)
$FOLLOWUP_FILE = "email_followup.txt";

# tmp file
$TMP_FILE = "/tmp/incident.out.$$";

# Signature to append to message
# If not defined, no signature
# $SIGNATURE = "/home/valankar/.sig";

# Favorite editor
$EDITOR = "vi";

# If you change the subject format below, keep DOMAIN_INS and INCNUM_INS in the subject
# or followup processing will not work
$attack_email_subject = "Possible attack from your system - DOMAIN_INS - Incident #: INCNUM_INS";

# The actual message that will be sent to the administrators
$attack_email = <<EOF;

System Administrators, 

        We noticed what is most likely an attempted attack on our  
computers which has been logged from the address:

FQDN_INS
IP_INS

        We ask that you please look into this matter as soon as
possible.  Attached below is our logs which show evidence of the attack.
The date and times of the requests are listed below.  Please let us know 
how we can be of further assistance tracking this down. We consider this 
a very serious threat to our company systems. Thank you for your 
cooperation in this matter.

Note that all hostnames listed below are all in the $MYDOMAIN domain,
and all times are $TIMEZONE_STR.

Please keep the Subject of this message intact on all correspondence.

SIG_INS
Date of this report: DATE_INS
DESC_INS
EOF

# If an attacker appears to be resolved to one of these domains, we should
# ignore this domain entry. They are registrars, and they will complain
# that they are not authority for security incidents.
# Note: These are ADDED to @stock_bogus_domains
@bogus_domains = ();

# END CONFIG SECTION
####################

# Bogus domains
# Note: These are the stock ones that I know of. You can add more 
# via @bogus_domains
@stock_bogus_domains = (
	"cnnic.net.cn",
	"connect.com.au",
	"dns.br",
	"krnic.net",
	"nic.br",
	"nic.or.kr",
	"nic.ad.jp",
	"aunic.net",
	"apnic.net",
	"registro.br",
	"ripe.net",
	"telstra.net"
);

# Structure defining patterns to look for in IP lookups, and where to be redirected to
# Format:
# Toplevel
#   Regex patterns
#     Whois server
#     Additional emails
#     Regex patterns
#       Whois server
#       ...
#   Regex patterns
#     ...
#
# This is processed recursively so it can be any depth. Only the first pattern match
# that is found is used and the rest ignored. The pattern matches aren't guaranteed 
# to be processed in the order specified below.
#
# The toplevel patterns are used on ARIN's rwhois response (NOT whois), and the rest
# apply to their parent defined whois (NOT rwhois) server.
#
$arin_redirects = {
	'whois.ripe.net' => { 
		server => 'whois.ripe.net' 
	},
	'(?i)whois.apnic.net' => { 
		server => 'whois.apnic.net',
		patterns => {
			'KRNIC' => {
				server => 'whois.nic.or.kr'
			},
			'TELSTRA' => {
				server => 'whois.telstra.net'
			},
			'CONNECT' => {
				server => 'whois.connect.com.au'
			}
		}
	},
	'NETBLK-BRAZIL' => {
		server => 'whois.registro.br',
		add_mails => 'nbso@nic.br'
	},
	'Japan Network Information Center' => {
		server => 'whois.nic.ad.jp'
	}
};

# If domain whois fails on server that are keys of this hash, try
# all of the listed possible servers. These are mainly exceptions, and
# I've found only one case so far that requires this: bigpond.net.au.
# ARIN's rwhois refers this to whois.aunic.net, but this server does not
# have any information. The actual whois information is in whois.connect.com.au,
# which was determined from http://www.aunic.net. Sigh...
#
$referral_possibilities = {
	'whois.aunic.net' => [
		'whois.connect.com.au',
		'whois.telstra.net'
	]
};

# When digging for a domain that is part of bogus_domains, sometimes
# we should go to another whois server. The following provides some hints.
# I've only found one IP in my experience that requires this: 210.99.62.8
# If you try to look this up in ARIN, you get sent to APNIC, but APNIC
# has no information. However, when you dig this IP, you see that it is
# krnic.net, which has its own whois server.
#
# Note: as of 10/14/2001 the above IP works without these hints. So this may
# be removed soon.
%stock_whois_hints = (
	"krnic.net" => "whois.krnic.net"
);

sub catch_ctrlc {
	my $signame = shift;

	if (length($curr_rep_dir) && length($REP_DIR) && length($domain)) {
		print("\nRemoved $curr_rep_dir\n") if (-d $curr_rep_dir && rmdir($curr_rep_dir));
		print("Removed $REP_DIR/$domain\n") if (-d "$REP_DIR/$domain" && rmdir("$REP_DIR/$domain"));
	}
	unlink($TMP_FILE);
	warn("Quitting on SIG$signame.\n");
	exit(1);
}

# Find what seems to be an attack
# This is determined by finding out the most number of occurrences of an IP
# address in the input
sub get_suspects {
	my $max;
	my %ips;

	foreach (@_) {
		if (/(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/) {
			# Ignore IPs that syslog appears to be from?
			# next if (/^\S+\s+\S+\s+\S+\s+$1/);
			$ips{$1}++;
			$max = $1 if ($ips{$1} > $ips{$max});
		}
	}
	if ($opts{'a'} || $opts{'A'}) {
		return (sort { $ips{$b} <=> $ips{$a} } keys %ips);
	} else {
		return ($max);
	}
}

sub reverse_lookup {
	my @addrs;

	if ($_[0] =~ /^\d/) {
		return (gethostbyaddr(inet_aton($_[0]), AF_INET));
	} else {
		(undef, undef, undef, undef, @addrs) = gethostbyname($_[0]);
		if (@addrs) {
			return(inet_ntoa($addrs[0]));
		} else {
			return "";
		}
	}
}

sub isolate_input {
	my $rev = pop(@_);
	my $ip = pop(@_);
	my @result;

	foreach (@_) {
		push(@result, $_) if (/$ip/ || ($rev && /$rev/));
	}

	return @result;
}

# Much of this is take from prwhois.pl which comes with Net::Rwhois
sub get_rwhois {
	my @result;
	my $client;
	my $rs;
	my $obj;
	my $attr;
	my $sresult;
	my $func;
	my $query;

	$query = (split(/\s+/, $_[0]))[1];
	$client = new Net::Rwhois(Host => $_[1] ? $_[1] : $RWHOIS_SERVER, Port => $RWHOIS_PORT);

	for (my $i = 0; $i < $RWHOIS_NUM_TRIES; $i++) {
		eval {
			local $SIG{ALRM} = sub {
				print("timeout,retrying..") if ($i < ($RWHOIS_NUM_TRIES - 1));
			};
			alarm($RWHOIS_TIMEOUT);
			if ($_[2]) {
				$rs = $client->execute_query(Query_String => $_[0]);
			} else {
				$rs = $client->recursive_execute_query(Query_String => $_[0]);
			}
			alarm(0);
		};
		last if ($rs);
		sleep($RWHOIS_RETRY_INTERVAL);
	}
	return () if (!$rs);

	foreach $obj ($rs->get_objects()) {
		if (ref($obj) =~/RwhoisObject/io) {
			foreach $attr ($obj->get_attribute_names()) {
				foreach $value ($obj->get_attribute($attr)) {
					$sresult .= sprintf("%-20s%s\n", $attr . ":", $value);
				}
			}
		} elsif (ref($obj) =~ /ReferralSet$/io) {
			foreach ($obj->get_all_referrals()) {
				$sresult .= "$obj\n";
			}
		}
	}
	foreach $r ($rs->get_referrals()) {
		my $proto = $r->get_protocol();
		next if ($proto eq "rwhois");
		if ($proto eq "whois") {
			my $server = $r->get_url();

			$server =~ s/.*\///;
			foreach ($server, @{$referral_possibilities->{$server}}) {
				print("\n  redirect to $_...");
				last if ((@result = get_standard_whois($query, $_)));
			}
		}
	}
	@result = (@result, split(/\n/, $sresult));
	foreach (@result) {
		$_ .= "\n" if (!(/\n/));
	}

	# Add an extra blank if one is not already there
	push(@result, "\n") if ($result[$#result] !~ /^\s+$/);

	return valid_whois_result($query, @result) ? @result : ();
}

sub get_allwhois {
	my @result;
	my $work_addr = $_[0];
	my $likely_domain = "";
	my @candidates;

	# Example hostname: reality.cse.fau.edu.co
	# We have to try a whois on all extensions until we get info:
	#   cse.fau.edu.co
	#   fau.edu.co
	#   edu.co
	# 
	# We really will not try a whois on 'co' only, however I have seen domains
	# like this (www.ni for example)

	# Find the most likely domain, i.e. one with an MX record
	print("Searching MX records for likely candidate...");
	while ($work_addr =~ /.+\..+/) {
		if (!$queried{lc($work_addr)}) {
			if (!$likely_domain && dig_mx_ip($work_addr)) {
				$likely_domain = $work_addr;
			} else {
				push(@candidates, $work_addr);
			}
		}

		# Strip out first prefix for next iteration
		$work_addr =~ s/.*?\.//;
	}
	print("done.\n");
	@candidates = reverse(@candidates);
	if ($likely_domain) {
		push(@candidates, $likely_domain);
		print("MX record for $likely_domain found.\n");
	}

	while (!@result && ($work_addr = pop(@candidates))) {
		if (!$queried{lc($work_addr)}) {
			print("RWHOIS $work_addr...");
			@result = get_rwhois("domain $work_addr");
			if (!@result) {
				print("failed.\n");
			} else {
				print("succeeded.\n");

				# Use the domain that has an MX record regardless of which responds to whois
				if ($likely_domain && ($work_addr ne $likely_domain) && !dig_mx_ip($work_addr)) {
					print("No MX found for $work_addr, assuming domain $likely_domain.\n");
					@result = ("$likely_domain\n", @result);
				} else {
					@result = ("$work_addr\n", @result);
				}
			}
			$queried{lc($work_addr)}++;
		}

		# Strip out first prefix for next iteration
		$work_addr =~ s/.*?\.//;
		sleep(1);
	}

	return @result;
}

# Should probably make this better
sub valid_whois_result {
	my $query = shift;
	my $numlines = scalar(@_);

	return 0 if (($numlines < 3) || grep(/(^invalid query|^No entries found in the.*database.|Error:|name is not registered|\% No entries found|^No match!!)/, @_));

	# aunic failures
	# look for all blank or comment lines
	return 0 if (($numlines <= 7) && (grep(/^(\%.*|\s*)$/, @_[1..$#_]) == ($numlines - 1)));

	# Some registrars (China) have hosts themselves registered, we don't want these
	return 0 if (($numlines <= 11) && grep(/^host:\s*$query\s*$/i, @_));

	# whois.cira.ca returns info for subdomains, ignore them
	return 0 if (grep(/^Status:\s+UNAV\s*$/, @_) && !get_emails(@_));

	return 1;
}

sub get_arin_contacts {
	my @result;
	my %cqueried;

	foreach (@_) {
		if (/\w+-contact:\s*(\S+)/) {
			next if ($cqueried{lc($1)});
			# Add an extra blank if one is not already there
			push(@result, "\n") if ($result[$#result] !~ /^\s+$/);
			@result = (@result, get_rwhois("contact $1", $RWHOIS_ARIN_SERVER));
			$cqueried{lc($1)}++;
		}
	}
	return @result;
}

# Recursive routine to process the regexp-to-whois-server redirects
sub process_arin_redirect {
	my $query = shift;
	my $href = shift;
	my $aref = shift;
	my $found = 0;
	my $lookfor;

	return 0 if (!@{$aref});

	foreach (keys %{$href}) {
		$lookfor = $_;

		if (grep(/$lookfor/, @{$aref})) {
			if (defined($href->{$_}->{'server'})) {
				print("\n  redirect to " . $href->{$_}->{'server'} . "...");
				@{$aref} = get_standard_whois($query, $href->{$_}->{'server'});

				# Add extra emails if necessary
				foreach (split(/,/, $href->{$_}->{'add_mails'})) {
					push(@extra_addrs, $_);
				}
			}
			if (defined($href->{$_}->{'patterns'})) {
				$found += process_arin_redirect($query, \%{$href->{$_}->{'patterns'}}, $aref);
			}
			$found++ if (@{$aref});
		}
		last if ($found);
	}
	return $found;
}

sub get_arin_whois {
	my @result;
	my $work_ip = $_[0];
	my $orig_ip = $_[0];
	my $loopcount = 0;
	my @clines;

	until (@result) {
		last if ($queried{lc($work_ip)});

		$queried{lc($work_ip)}++;
		print("RWHOIS $work_ip...");
		@result = get_rwhois("network $work_ip", $RWHOIS_ARIN_SERVER, 1);

		# Take care of common redirections
		# These strings are matched in the RWHOIS reply
		if (!process_arin_redirect($work_ip, $arin_redirects, \@result)) {
			# Process ARIN rwhois output
			# Isolate to closest netblock
			if ((grep(/^class-name:/, @result) > 1)) {
				print("Isolating likely netblock...");
				@result = prune_arin_netblock($work_ip, @result);
			}
			# Gather contacts
			if ((@clines = grep(/^\w+-contact:/, @result))) {
				print("Getting ARIN referred contacts...");
				@result = (@result, get_arin_contacts(@clines));
			}
		}
		# Sleep a little bit in case of infinite loop
		sleep(1);
		@result = () if (@result < 2);
		last if ($loopcount++ > 10);
	}

	if (@result) {
		print("succeeded.\n");
	} else {
		print("failed.\n");
	}
	return @result;
}

sub prune_arin_netblock {
	my $ip = shift;
	my @classrefs;
	my $i = -1;
	my $probable_index;
	my $best_diff;

	# Split up netblocks into array of arrays
	foreach (@_) {
		$i++ if (/^class-name:/);
		next if ($i < 0);
		push(@{$classrefs[$i]}, $_);

		if (/^ip-network-block:\s+(\d+\.\d+\.\d+\.\d+)\s+-\s+(\d+\.\d+\.\d+\.\d+)/) {
			my $diff = 0;
			@begspl = split(/\./, $1);
			@ipspl = split(/\./, $ip);
			for (my $j = 0; $j < @ipspl; $j++) {
				$diff += $ipspl[$j] - $begspl[$j];
			}
			if (!defined($best_diff) || ($diff < $best_diff)) {
				$best_diff = $diff;
				$probable_index = $i;
			}
		}
	}

	if (!defined($probable_index)) {
		return @_;
	} else {
		return @{$classrefs[$probable_index]};
	}
}

sub get_standard_whois {
	my @result;
	my $pid;
	my $cmd;

	for (my $i = 0; $i < $WHOIS_NUM_TRIES; $i++) {
		eval {
			local $SIG{ALRM} = sub {
				kill(9, $pid);
				print("timeout,retrying..") if ($i < ($WHOIS_NUM_TRIES - 1));
			};

			($cmd = $WHOIS_FORMAT) =~ s/SERVER/$_[1]/g;
			$cmd =~ s/QUERY/$_[0]/g;

			$pid = open(WHOIS, "$cmd 2>&1 |") or die("Can't fork: $!\n");
			alarm($WHOIS_TIMEOUT);
			@result = <WHOIS>;
			alarm(0);
		};
		close(WHOIS);
		if (@result < 3) {
			if (grep(/connection refused/i, @result)) {
				print("connection refused,retrying..") if ($i < ($WHOIS_NUM_TRIES - 1));
				@result = ();
			} elsif (grep(/(unknown host|host unknown)/, @result)) {
				return ();
			}
		}
		last if (@result);
		sleep($WHOIS_RETRY_INTERVAL);
	}

	return valid_whois_result($_[0], @result) ? @result : ();
}

sub get_whois {
	my @result;

	return () if (!($_[0]));

	# Use specified whois server if we were given one
	if ($_[1]) {
		@result = get_standard_whois($_[0], $_[1]);
	} elsif ($_[0] =~ /^\d+\.\d+\.\d+\.\d+$/) {
		# It's an IP, so use ARIN
		@result = get_arin_whois($_[0]);
	} else {
		@result = get_allwhois($_[0]);
	}

	return @result;
}

# Return a list of possible email addresses
sub get_emails {
	my @emails;
	my %ignore_emails;
	my $email;
	my @unlikely_emails;
	
	foreach (@_) {
		if (/(\S+\@\S+\.[A-Za-z]+\S*)/) {
			($email = lc($1)) =~ s/mailto://g;
			$email =~ s/[^\@\w\.\-]//g;

			# Some Korean registrars have --<digits>-- appended to the email for some reason
			$email =~ s/-+\d+-+$//;

			# Sometimes emails end with punctuation characters
			$email =~ s/\W+$//;

			if (check_domain($email)) {
				# Emails beginning with 'changed:' should be ignored UNLESS nothing
				# else is found
				if (/^(changed:|updated-by:)/i) {
					push(@unlikely_emails, $email);
				} else {
					push(@emails, $email);
				}
			} else {
				print("Email $email is marked as to be ignored.\n") if (!$ignore_emails{$email});
				$ignore_emails{$email}++;
			}
		}
	}
	@emails = @unlikely_emails if (!@emails);
	return (@emails);
}

sub host_probe {
	my @result;
	my $conn_status;
	my @answ_ports;

	foreach (@tcp_ports) {
		$conn_status = tcp_connect($_[0], $_);
		if ($conn_status) {
			push(@result, $conn_status);
			push(@answ_ports, $_);
		}
	}

	# handle any operations for special ports
	foreach (@answ_ports) {
		push(@result, port_special($_[0], $_, $_[1]));
	}
	return @result;
}

sub port_special {
	my $ip = $_[0];
	my $port = $_[1];
	my $storage = $_[2];
	my $result;

	if ($port == 139) {
		$result .= probe_smbclient($ip);
	} elsif ($port == 79) {
		$result .= probe_finger($ip);
	} elsif ($port == 80) {
		$result .= probe_http($ip);
	} elsif ($port >= 6000 && $port <= 6010) {
		$result .= probe_xwd($ip, $port, $storage);
	}

	return $result;
}

sub probe_xwd {
	my $result;
	my $dispnum;
	my $cmd;
	my $show_str;

	$dispnum = $_[1] - 6000;

	$show_str = "XWD SCREENDUMP OF $_[0]:$dispnum";
	print $show_str;

	$cmd = "xwd -display $_[0]:$dispnum -out $_[2]/$_[0].xwd -root 2>&1";

	$result = "$show_str$cmd\n";
	$result =~ s/-out .*\.xwd/-out $_[0].xwd/;
	$result .= `$cmd` . "\n";

	if ($? >> 8) {
		$result = "";
		print(" failed\n");
	} else {
		print(" succeeded\n");
	}
	return $result;
}
	
sub probe_smbclient {
	my $result;
	my $show_str;

	$show_str = "SMBCLIENT SHARE LIST OF $_[0]\n";
	print $show_str;

	$result = `smbclient -N -L $_[0] 2>/dev/null`;
	$result =~ s/^[Aa]dded interface.*//gm;
	$result =~ s/^session request to .* failed.*//gm;
	$result =~ s/^\n//gm;

	$result = "$show_str$result" if ($result);
	return $result;
}

sub probe_finger {
	my $output;
	my $show_str;

	$show_str = "FINGER \@$_[0]\n";
	print $show_str;

	$output = `finger \@$_[0] 2>&1`;
	
	$output = "$show_str$output\n" if ($output);
	return $output;
}

sub probe_http {
	my $output;
	my $show_str;

	$show_str = "HTTP GET / TO $_[0]\n";
	print $show_str;
	$output = `wget -q -O - $_[0] 2>&1`; 
	$output = "$show_str$output\n\n" if ($output);
	return $output;
}

sub tcp_connect {
	my $ip = $_[0];
	my $port = $_[1];
	my $line;
	my $result;
	my $obj;
	my $show_str;
	my $timeout;

	$show_str = "TCP CONNECT TO $ip:$port";
	print $show_str;

	$result = "$show_str succeeded\n";

	$obj = new Net::Telnet (Errmode => "return");
	$obj->open(Host => $ip, Port => $port, Timeout => $TCP_TIMEOUT);

	$timeout = $tcp_override_timeouts{$port};
	$timeout = $TCP_TIMEOUT if (!$timeout);
	while (!$obj->errmsg && ($line = $obj->get(Timeout => $timeout))) {
		$result .= $line;
		$timeout = $TCP_TIMEOUT;
	}

	$obj->close();

	if ($obj->errmsg() && $obj->errmsg !~ /^read timed-out/) {
		print(" failed\n");
		$result = "";
	} else {
		$result .= "\n" if ($result !~ /\n$/);
		$result .= "Connection closed.\n\n";
		print(" succeeded\n");
	}
	return $result;
}

sub get_y_or_n {
	my $answer;

	return 0 if ($batch_mode);

	print $_[0];
	$answer = <STDIN>;
	if ($answer =~ /^y/i) {
		return 1;
	}
	if ($_[1] && $answer =~ /^\s*$/) {
		return 1;
	}
	return 0;
}

sub get_description {
	my @desc;

	if (get_y_or_n("Add description of attack to message? ")) {
		print("Enter description (ctrl-d on blank line to end):\n");
		@desc = <STDIN>;
		chop(@desc);
	}
	return @desc;
}

sub dig_mx_ip {
	my $result;
	my $domain = shift;
	my @digout;
	my $ip = "";
	my $mx = "";

	@digout = `dig $domain mx`;
	foreach (@digout) {
		if (/\S+\s+\S+\s+IN\s+A\s+(\d+\.\d+\.\d+\.\d+)/) {
			$ip = $1;
			last;
		}
		if (/\S+\s+\S+\s+\S+\s+MX\s+\S+\s+(\S+)/) {
			($mx = $1) =~ s/\.\s*$//;
		}
	}
	
	# Perform reverse lookup of MX record if address was not returned
	$ip = reverse_lookup($mx) if (!$ip && $mx);

	return $ip;
}

sub dig_domain {
	my $result;
	my $work_ip = $_[0];
	my @digout;

	while (!$result && $work_ip =~ /\d+\.\d+/) {
		@digout = `dig -x $work_ip`;
		$look_for = join('.', reverse(split(/\./, $work_ip))) . ".in-addr.arpa.";
		if (@found = grep(/^$look_for/i, @digout)) {
			if ($found[0] =~ /\s+SOA\s+(\S+)/) {
				# It's an authoritative line
				$result = $1;
			} else {
				my @tmparr = split(/\s+/, $found[0]);
				$result = $tmparr[$#tmparr];
			}
			$result =~ s/\.$//;
			if ($result !~ /in-addr\.arpa$/) {
				return($result);
			} else {
				$result = "";
			}
		}

		# Strip off end part of IP and try again
		$work_ip =~ s/\.\d+?$//;
	}

	return "";
}

sub dig_whois {
	my $domain;
	my @result;

	$domain = dig_domain($_[0]);
	if ($domain) {
		$domain = lc($domain);
		print("Possible candidate: $domain\n");
		if (check_domain($domain)) {
			@result = get_whois($domain);
		} else {
			print("$domain is marked as to be ignored.\n");
			# If domain has a whois hint, try it
			$hint = whois_hint($domain);
		}
	}

	return @result;
}

sub check_domain {
	my $domain = shift;

	foreach (@bogus_domains, @stock_bogus_domains) {
		return 0 if (lc($domain) =~ /$_$/);
	}
	return 1;
}

sub whois_hint {
	my $domain = shift;
	
	foreach (keys %stock_whois_hints) {
		return $stock_whois_hints{$_} if ($domain =~ /$_/);
	}

	return "";
}

sub parse_hints {
	my $parse;

	$parse = join('', @input);
	die("Mail header found, possible loop. Quitting.\n") if ($parse =~ /^$mail_header/);
	if ($parse =~ s/^inc_hint: (.*)$//gm) {
		$isolate = $1;
	}
	if ($parse =~ s/^inc_desc: (.*)$//gm) {
		@desc = ($1);
	}
}
	
sub email_pipe_followup {
	my $lookfor;
	my $input;
	my $domain;
	my $incnum;
	my $dest_file;
	my $incopy;
	
	print("Analyzing piped input as email followup...\n");
	$lookfor = $attack_email_subject;
	$lookfor =~ s/DOMAIN_INS/\(\\S*)/;
	$lookfor =~ s/INCNUM_INS/\(\\d\{14,\}\)/;
	$lookfor =~ s/\s+/\\s+/g;
	$input = join('', <STDIN>);
	($incopy = $input) =~ s/^\s*>\s*//gm;
	if ($incopy !~ /$lookfor/m) {
		print STDERR ("Cannot determine what input email is in reference to. Quitting.\n");
		print STDERR ("Regular expression was:\n$lookfor\n");
		return 1;
	} else {
		$domain = $1;
		$incnum = $2;
	}

	$input = "From XXX " . strftime("%a %b %e %H:%M:%S %Z %Y", localtime) . "\n" . $input 
		if ($input !~ /^From /);
	$dest_file = "$REP_DIR/$domain/$incnum/$FOLLOWUP_FILE";
	if (!open(OFILE, ">>$dest_file")) {
		print STDERR ("Cannot create $dest_file: $!\n");
		return 1;
	}
	print OFILE $input, "\n";
	close(OFILE);
	print("Wrote email to $dest_file\n");
	return 0;
}

sub load_configuration {
	my @config = ( $ENV{HOME} . "/.incident.pl.cfg", "./incident.pl.cfg" );
	my @pwent;

	if ((@pwent = getpwuid($>))) {
		push(@config, $pwent[7] . "/.incident.pl.cfg");
	}
	foreach (@config) {
		if (-f $_) {
			print("Using config $_\n");
			require $_;
			last;
		}
	}
}

sub head {
	my @retval;

	for (my $i = 0; $i < 5; $i++) {
		push(@retval, $_[$i]);
	}
	return @retval;
}

sub usage {
	print("Usage: ./incident.pl [-x <FQDN, IP, or DOMAIN>] [-AaehH] [<inputfile> [<isolatestring>]]\n\n");
	print("-a Process ALL IPs found in input, in order of their occurrence\n");
	print("\tPrompt before each IP\n");
	print("-A Process ALL IPs found in input, in order of their occurrence\n");
	print("\tDo NOT prompt before each IP\n");
	print("-e Email followup processing (all other options/arguments ignored)\n");
	print("-h Display this help\n");
	print("-H Do not perform host probe\n");
	print("-x Plain contact gathering\n\n");
	exit(1);
}

#### Start

$SIG{INT} = \&catch_ctrlc;
$| = 1;
$batch_mode = 0;
$mail_header = "X-Sender: incident.pl";

getopts('AabehHx', \%opts);
usage() if ($opts{'h'});

load_configuration();

# Email piping
if ($opts{'e'}) {
	exit(email_pipe_followup());
} elsif ($opts{'b'}) {
	$batch_mode = 1;
}

# Just contact gathering
if ($opts{'x'}) {
	$lookup_only = 1;
	@suspects = ($ARGV[0]);
} else {
	print("Analyzing input...\n");
	# Take care of any isolations specified
	if (@ARGV == 2) {
		$isolate = $ARGV[1];
		pop(@ARGV);
	}

	@input = <>;

	if ($batch_mode) {
		parse_hints();
	}

	if ($isolate) {
		print("Isolating search to $isolate\n");
		$isolate =~ s/\./\\\./g;
		@suspinput = grep(/$isolate/, @input);
	} else {
		@suspinput = @input;
	}
	@suspects = get_suspects(@suspinput);
}

$REP_DIR = `pwd` if (!$REP_DIR);
chomp($REP_DIR);

foreach (@suspects) {
	# Undefine any globals
	undef %queried;
	undef $hint;
	undef %saw;
	undef @extra_addrs;
	undef @whois;
	undef @whois_rev;

	# If processing all IPs, get confirmation
	if ($opts{'a'}) {
		my $answer;
		my $ip = $_;

		$ip =~ s/\./\\\./g;
		print("\nExcerpt of logs:\n", head(grep(/$ip/, @input)), "\n");
		print("Process IP $_ ? ([y]/n/q) ");
		chomp($answer = <STDIN>);
		last if ($answer =~ /^q/i);
		next if ($answer =~ /^n/i);
	}
		
	print("Attack appears to be from $_\n");
	print("Reverse lookup: ");
	printf("%s\n", ($rev = reverse_lookup($_)) ? $rev : "failed");
	$rev = "" if ($rev eq $_);

	# Swap the values if $_ is not an IP
	if ($_ !~ /^\d+\.\d+\.\d+\.\d+$/) {
		($_, $rev) = ($rev, $_);
	}

	if (!$lookup_only) {
		print("Isolating input for $_" . ($rev ? " and $rev" : "") . "\n");
		@newlogs = isolate_input(@input, $_, $rev);
	}

	$start_time = time();
	$now_string = strftime("%a %b %e %H:%M:%S %Z %Y", localtime($start_time));
	$dirname = strftime("%Y%m%d%H%M%S", localtime($start_time));

	print("\nInformation Gathering\n\n$now_string\n\n");

	if (!$_ && $rev) {
		print("No IP known. Digging MX record for $rev to find an IP...");
		if (($_ = dig_mx_ip($rev))) {
			print("found $_.\n");
		} else {
			print("failed.\n");
		}
	}

	@whois = get_whois($_);
	@whois_rev = get_whois($rev) if ($rev);

	# If reverse lookups fail, dig for the domain
	if (!$whois_rev[0]) {
		print("Trying to figure out domain name after failed reverse lookup or whois.\n");
		@whois_rev = dig_whois($_);
	}

	chop($domain = $whois_rev[0]);
	if (!$domain) {
		print("Failed to determine any domain.\n");
		if ($hint) {
			print("Trying hint $hint\n");
			@whois_rev = get_whois($_, $hint);
		}
	}

	@emails = (get_emails(@whois, @whois_rev), @extra_addrs);
	die("No email contacts found. Quitting.\n") if (!@emails);

	# If domain is still unknown, assume it is domain of the first email
	if (!$domain) {
		my @newwhois;
		my $newdomain;
		(undef, $domain) = split(/\@/, $emails[0]);
		print("Assumption made from first email contact that the domain may be $domain.\n");
		@newwhois = get_whois($domain);
		@whois = (@whois, @newwhois);
		@emails = (@emails, get_emails(@newwhois));
		chop($newdomain = $newwhois[0]);
		$domain = $newdomain if ($newdomain);
	}

	$domain = lc($domain);
	push(@emails, "$_\@$domain") foreach (@admin_emails);
	@emails = grep(!$saw{$_}++, @emails);

	if ($lookup_only) {
		print ("\n--- BEGIN CONTACT INFO ---\n\n");
		print @whois, "\n";
		print @whois_rev;
		print ("\nAssumed domain: $domain\n\n");
		print ("Possible admin contacts:\n");
		foreach (@emails) {
			print("$_\n");
		}
		exit(0);
	}

	$curr_rep_dir = "$REP_DIR/$domain/$dirname";
	system("mkdir -p $curr_rep_dir");
	system("touch $REP_DIR/$domain");

	@host_info = host_probe($_, $curr_rep_dir) unless ($opts{'H'});

	$out_msg = $attack_email;
	
	print("\nExcerpt of logs:\n", head(@newlogs), "\n");
	@desc = get_description(@newlogs) if (!@desc);

	$tmp_desc = "\nDescription of this attack:\n" . join("\n", @desc) .
		"\n" if (@desc);
	$out_msg =~ s/DESC_INS/$tmp_desc/;
	$out_msg =~ s/IP_INS/$_/g;
	$out_msg =~ s/FQDN_INS/$rev/g;
	$out_msg =~ s/DATE_INS/$now_string/g;

	$msg_subj = $attack_email_subject;
	$msg_subj =~ s/DOMAIN_INS/$domain/g;
	$msg_subj =~ s/INCNUM_INS/$dirname/g;

	$sig = "";
	$sig = `cat $SIGNATURE` if (-e $SIGNATURE);
	$out_msg =~ s/SIG_INS/$sig/g;

	open(FOUT, ">$curr_rep_dir/$REP_FILE") or die("Cannot create: $!\n");
	print FOUT $out_msg;
	print FOUT "Logs of attack:\n", @newlogs, "\n";
	print FOUT "Name: $rev\n" if ($rev);
	print FOUT "Address: $_\n\n";
	print FOUT @whois, "\n";
	print FOUT @whois_rev;
	print FOUT @host_info;
	close(FOUT);
	print("Email for $_ written to $curr_rep_dir/$REP_FILE\n");

	if (get_y_or_n("\nView/edit message? ")) {
		system("$EDITOR $curr_rep_dir/$REP_FILE");
	}

	open(FOUT, ">$curr_rep_dir/$SENT_TO_FILE") or warn("Cannot create: $!\n");
	print ("\nPossible admin contacts:\n");
	$mailto = "";
	foreach (@emails) {
		$mailto .= "$_,";
		print ("$_\n");
		print FOUT ("$_\n");
	}
	close(FOUT);
	$mailto =~ s/,$//;

	if (get_y_or_n("\nSend message? ") || $batch_mode) {
		my $body = `cat $curr_rep_dir/$REP_FILE`;
		my $sender = new Mail::Sender( { from => $FROM_ADDR,
			smtp => $SMTP_SERVER } ) or warn("$Mail::Sender::Error\n");
		$sender->MailMsg( { 
			to => $mailto, subject => $msg_subj, msg => $body, headers => $mail_header,
			cc => $ALWAYS_CC } ) or warn("$Mail::Sender::Error\n");
	}
}

unlink($TMP_FILE);
